==> rebotics-scripts-19.4.4.tar.gz: rebotics-scripts-19.4.4/scripts/retailer_api_client.py <==
import json
import os
from multiprocessing import Pool

import click
import requests
from prettytable import PrettyTable

from sdk.providers import RetailerProvider, ProviderHTTPClientException
from sdk.utils import download_file, mkdir_p, \
    get_filename_from_url
from .utils import format_processing_action_output


class ApiClientException(Exception):
    pass


@click.group()
@click.option('-r', '--retailer')
@click.option('-h', '--host')
@click.option('-u', '--user', prompt=True, )
@click.option('--password', prompt=True, hide_input=True, confirmation_prompt=False)
@click.option('-f', '--format', default='table', type=click.Choice(['table', 'id', 'json']))
@click.option('-v', '--verbose', is_flag=True, help='Enables verbose mode')
@click.version_option()
@click.pass_context
def api(ctx, retailer, host, user, password, format, verbose):
    """
    This action will login you into retailer that you provide.
    """
    ctx.ensure_object(dict)
    if host:
        provider = RetailerProvider(host=host, retries=1)
    else:
        if retailer is None:
            raise ValueError('You need to supply either host or retailer')
        else:
            response = requests.post('http://admin.rebotics.net/retailers/host/', data={
                'company': retailer
            }).json()
            provider = RetailerProvider(host=response['host'], retries=1)
    try:
        provider.api_login(user, password)
        if verbose:
            click.echo("Successfully logged in for retailer")
    except ProviderHTTPClientException:
        raise ProviderHTTPClientException("Failed To login")
    ctx.obj['provider'] = provider

    ctx.obj['format'] = format
    ctx.obj['verbose'] = verbose


@api.command()
@click.pass_context
def version(ctx):
    provider = ctx.obj['provider']
    click.echo(provider.version())


@api.command()
@click.option('-t', '--input_type')
@click.option('-s', '--store', type=click.INT)
@click.argument('files', nargs=-1, required=True, type=click.File('rb'))
@click.pass_context
def upload_files(ctx, input_type, store, files):
    provider = ctx.obj['provider']
    verbose = ctx.obj['verbose']

    file_ids = []
    for f_ in files:
        response = provider.processing_upload(
            store, f_, input_type
        )
        file_ids.append(response['id'])

        if verbose:
            click.echo(response)  # redirecting this output to stderr
    click.echo(' '.join(map(str, file_ids)))


REQUEUE_TYPES = {
    "facenet_kf": 'requeue_for_facenet_keyframes_key',
    "pt_multiclass": 'REQUEUE_PRICE_TAGS_DETECTION_MULTICLASS',
    "pt_heatmap": 'REQUEUE_PRICE_TAGS_DETECTION_MULTICLASS_HEATMAP',
    "pt_voting": 'REQUEUE_PRICE_TAGS_DETECTION_MULTICLASS_VOTING',
}


def requeue_task(d):
    provider = d['provider']
    response = provider.requeue(d['id'], d['requeue_type'])
    if d['verbose']:
        click.echo(response)
        click.echo(response['id'])
    return response


@api.command()
@click.argument('processing_ids', required=True, nargs=-1, type=click.INT)
@click.option('-t', '--requeue-type', type=click.Choice(choices=REQUEUE_TYPES.keys()), required=False, default=None)
@click.option('-c', '--concurrency', type=int, default=4)
@click.pass_context
def requeue(ctx, processing_ids, requeue_type, concurrency):
    provider = ctx.obj['provider']
    verbose = ctx.obj['verbose']
    task_arguments = []

    for id_ in processing_ids:
        if verbose:
            click.echo('Requeue for %d' % id_)

        task_arguments.append({
            'verbose': verbose,
            'provider': provider,
            'id': id_,
            'requeue_type': REQUEUE_TYPES.get(requeue_type)
        })

    pool = Pool(concurrency)
    actions_list = pool.map(requeue_task, task_arguments)

    format_processing_action_output(actions_list, ctx.obj['format'])


@api.command()
@click.option('-t', '--input_type')
@click.option('-s', '--store', type=click.INT)
@click.option('-p', '--store-planogram', type=click.INT)
@click.option('--aisle')
@click.option('--section')
@click.option('-l', '--lens-used', is_flag=True, default=False)
@click.argument('files', nargs=-1, required=True, type=click.INT)
@click.pass_context
def create_processing_action(ctx, input_type, store, store_planogram, aisle, section, lens_used, files):
    provider = ctx.obj['provider']
    verbose = ctx.obj['verbose']

    response = provider.create_processing_action(
        store, files, input_type,
        store_planogram=store_planogram,
        aisle=aisle,
        section=section,
        lens_used=lens_used
    )
    if verbose:
        click.echo(response, )
    click.echo(response['id'])


def download_file_from_dict(d):
    click.echo('>> Downloading file into %s' % d['filepath'], err=True)
    result = download_file(d['url'], d['filepath'])
    click.echo('<< Downloaded file into %s' % d['filepath'], err=True)
    return result


@api.command()
@click.argument('actions', nargs=-1, required=True, type=click.INT)
@click.option('-t', '--target', type=click.Path(), default='.')
@click.option('-c', '--concurrency', type=int, default=4)
@click.pass_context
def download_processing_action(ctx, actions, target, concurrency):
    provider = ctx.obj['provider']
    verbose = ctx.obj['verbose']

    files_to_download = []

    for action_id in actions:
        processing_action_folder = os.path.join(target, 'ProcessingAction#%d' % action_id)

        if verbose:
            click.echo('Get Processing action %s' % action_id)
        data = provider.processing_action_detail(action_id)
        mkdir_p(processing_action_folder)
        results = os.path.join(processing_action_folder, 'results')
        inputs = os.path.join(processing_action_folder, 'inputs')

        mkdir_p(results)
        mkdir_p(inputs)

        for key in ['merged_image_jpeg', 'merged_image', ]:
            files_to_download.append({
                'url': data[key],
                'filepath': os.path.join(results, get_filename_from_url(data[key]))
            })

        for input_object in data.get('inputs', []):
            files_to_download.append({
                'filepath': os.path.join(get_filename_from_url(input_object['file'])),
                'url': input_object['file']
            })

        with open(os.path.join(processing_action_folder, 'processing_action_%d.json' % action_id), 'w') as fout:
            json.dump(data, fout, indent=4)

        if verbose:
            click.echo('Downloading files for %s' % (action_id,))

    pool = Pool(concurrency)
    pool.map(download_file_from_dict, files_to_download)

    if verbose:
        click.echo('Processing download success')


def upload_preview_task(d):
    provider = d['provider']
    images_path = d['images_path']
    upc = d['upc']
    delete = d['delete']

    if delete:
        try:
            response = provider.delete_product_previews(upc)
            click.echo('Deleted %s previews for product %s' % (response['deleted'], upc,), err=True)
        except ProviderHTTPClientException:
            click.echo('Product with UPC %s does not exists' % upc, err=True)

    for image_path in images_path:
        with open(image_path, 'rb') as image:
            try:
                provider.upload_product_preview(upc, image)
            except ProviderHTTPClientException as e:
                click.echo("xxx Failed to send image: %s" % image_path)
                click.echo(str(e), err=True)
            click.echo('<< Uploaded product preview for %s from %s' % (upc, image_path), err=True)


@api.command()
@click.option('-d', '--delete', is_flag=True)
@click.option('-c', '--concurrency', type=int, default=4)
@click.argument('path', type=click.Path(exists=True), default=os.getcwd())
@click.pass_context
def upload_previews_from_folder(ctx, delete, concurrency, path):
    """
    Upload previews from file system to the server in parallel
    It has increased retries
    """
    provider = ctx.obj['provider']
    provider.retries = 5
    verbose = ctx.obj['verbose']

    tasks = []
    for label in os.listdir(path):
        upc_folder = os.path.join(path, label)
        if not os.path.isdir(upc_folder):
            continue

        if verbose:
            click.echo('Reading folder: %s' % upc_folder)
        task = {
            'provider': provider,
            'images_path': [],
            'delete': delete,
            'upc': label
        }

        if label.isdigit():
            for filename in os.listdir(upc_folder):
                image_path = os.path.join(upc_folder, filename)

                if os.path.isfile(image_path):
                    task['images_path'].append(image_path)

        if task['images_path']:
            tasks.append(task)

    if verbose:
        click.echo('Number of tasks: {}'.format(len(tasks)))

    p = Pool(concurrency)
    p.map(upload_preview_task, tasks)
    click.echo('Finished')


PROCESSING_STATUSES = {
    'created': 'action created',
    'done': 'done',
    'error': 'error',
    'interrupted': 'interrupted',
    'progress': "in progress",
}


@api.command()
@click.option('-s', '--store', type=click.INT)
@click.option('--status', type=click.Choice(PROCESSING_STATUSES.keys()))
@click.option('-p', '--page', type=click.INT, default=1)
@click.option('-r', '--page-size', type=click.INT, default=10)
@click.pass_context
def processing_actions(ctx, store, status, page, page_size):
    provider = ctx.obj['provider']
    verbose = ctx.obj['verbose']

    table = PrettyTable()
    table.field_names = ['#', 'id', 'store', 'user', 'status',
                         'created', 'last_requeue', 'last_requeue ago', 'time in queue']

    if verbose:
        click.echo('Getting list of processing actions')
    data = provider.get_processing_action_list(
        store,
        PROCESSING_STATUSES.get(status),
        page=page,
        page_size=page_size,
    )
    format_processing_action_output(data, ctx.obj['format'])

==> rebotics-scripts-19.4.11.tar.gz: rebotics-scripts-19.4.11/scripts/retailer_api_client.py <==
import json
import os
from multiprocessing import Pool

import click
import requests
from prettytable import PrettyTable

from sdk.providers import RetailerProvider, ProviderHTTPClientException
from sdk.utils import download_file, mkdir_p, \
    get_filename_from_url
from .utils import format_processing_action_output


class ApiClientException(Exception):
    pass


@click.group()
@click.option('-r', '--retailer')
@click.option('-h', '--host')
@click.option('-u', '--user', prompt=True, )
@click.option('--password', prompt=True, hide_input=True, confirmation_prompt=False)
@click.option('-f', '--format', default='table', type=click.Choice(['table', 'id', 'json']))
@click.option('-v', '--verbose', is_flag=True, help='Enables verbose mode')
@click.version_option()
@click.pass_context
def api(ctx, retailer, host, user, password, format, verbose):
    """
    This action will login you into retailer that you provide.
    """
    ctx.ensure_object(dict)
    if host:
        provider = RetailerProvider(host=host, retries=1)
    else:
        if retailer is None:
            raise ValueError('You need to supply either host or retailer')
        else:
            response = requests.post('http://admin.rebotics.net/retailers/host/', data={
                'company': retailer
            }).json()
            provider = RetailerProvider(host=response['host'], retries=1)
    try:
        provider.api_login(user, password)
        if verbose:
            click.echo("Successfully logged in for retailer")
    except ProviderHTTPClientException:
        raise ProviderHTTPClientException("Failed To login")
    ctx.obj['provider'] = provider

    ctx.obj['format'] = format
    ctx.obj['verbose'] = verbose


@api.command()
@click.pass_context
def version(ctx):
    provider = ctx.obj['provider']
    click.echo(provider.version())


@api.command()
@click.option('-t', '--input_type')
@click.option('-s', '--store', type=click.INT)
@click.argument('files', nargs=-1, required=True, type=click.File('rb'))
@click.pass_context
def upload_files(ctx, input_type, store, files):
    provider = ctx.obj['provider']
    verbose = ctx.obj['verbose']

    file_ids = []
    for f_ in files:
        response = provider.processing_upload(
            store, f_, input_type
        )
        file_ids.append(response['id'])

        if verbose:
            click.echo(response)  # redirecting this output to stderr
    click.echo(' '.join(map(str, file_ids)))


REQUEUE_TYPES = {
    "facenet_kf": 'requeue_for_facenet_keyframes_key',
    "pt_multiclass": 'REQUEUE_PRICE_TAGS_DETECTION_MULTICLASS',
    "pt_heatmap": 'REQUEUE_PRICE_TAGS_DETECTION_MULTICLASS_HEATMAP',
    "pt_voting": 'REQUEUE_PRICE_TAGS_DETECTION_MULTICLASS_VOTING',
}


def task_requeue_processing(d):
    provider = d['provider']
    response = provider.requeue(d['id'], d['requeue_type'])
    if d['verbose']:
        click.echo(response)
        click.echo(response['id'])
    return response


def task_cancel_processing(d):
    provider = d['provider']
    response = provider.cancel(d['id'])
    if d['verbose']:
        click.echo(response)
        click.echo(response['id'])
    return response


@api.command()
@click.argument('processing_ids', required=True, nargs=-1, type=click.INT)
@click.option('-t', '--requeue-type', type=click.Choice(choices=REQUEUE_TYPES.keys()), required=False, default=None)
@click.option('-c', '--concurrency', type=int, default=4)
@click.pass_context
def requeue(ctx, processing_ids, requeue_type, concurrency):
    provider = ctx.obj['provider']
    verbose = ctx.obj['verbose']
    task_arguments = []

    for id_ in processing_ids:
        if verbose:
            click.echo('Requeue for %d' % id_)

        task_arguments.append({
            'verbose': verbose,
            'provider': provider,
            'id': id_,
            'requeue_type': REQUEUE_TYPES.get(requeue_type)
        })

    pool = Pool(concurrency)
    actions_list = pool.map(task_requeue_processing, task_arguments)

    format_processing_action_output(actions_list, ctx.obj['format'])


@api.command()
@click.argument('processing_ids', required=True, nargs=-1, type=click.INT)
@click.option('-c', '--concurrency', type=int, default=4)
@click.pass_context
def cancel(ctx, processing_ids, concurrency):
    provider = ctx.obj['provider']
    verbose = ctx.obj['verbose']
    task_arguments = []

    for id_ in processing_ids:
        if verbose:
            click.echo('Requeue for %d' % id_)

        task_arguments.append({
            'verbose': verbose,
            'provider': provider,
            'id': id_,
        })

    pool = Pool(concurrency)
    actions_list = pool.map(task_cancel_processing, task_arguments)

    format_processing_action_output(actions_list, ctx.obj['format'])


@api.command()
@click.option('-t', '--input_type')
@click.option('-s', '--store', type=click.INT)
@click.option('-p', '--store-planogram', type=click.INT)
@click.option('--aisle')
@click.option('--section')
@click.option('-l', '--lens-used', is_flag=True, default=False)
@click.argument('files', nargs=-1, required=True, type=click.INT)
@click.pass_context
def create_processing_action(ctx, input_type, store, store_planogram, aisle, section, lens_used, files):
    provider = ctx.obj['provider']
    verbose = ctx.obj['verbose']

    response = provider.create_processing_action(
        store, files, input_type,
        store_planogram=store_planogram,
        aisle=aisle,
        section=section,
        lens_used=lens_used
    )
    if verbose:
        click.echo(response, )
    click.echo(response['id'])


def download_file_from_dict(d):
    click.echo('>> Downloading file into %s' % d['filepath'], err=True)
    result = download_file(d['url'], d['filepath'])
    click.echo('<< Downloaded file into %s' % d['filepath'], err=True)
    return result


@api.command()
@click.argument('actions', nargs=-1, required=True, type=click.INT)
@click.option('-t', '--target', type=click.Path(), default='.')
@click.option('-c', '--concurrency', type=int, default=4)
@click.pass_context
def download_processing_action(ctx, actions, target, concurrency):
    provider = ctx.obj['provider']
    verbose = ctx.obj['verbose']

    files_to_download = []

    for action_id in actions:
        processing_action_folder = os.path.join(target, 'ProcessingAction#%d' % action_id)

        if verbose:
            click.echo('Get Processing action %s' % action_id)
        data = provider.processing_action_detail(action_id)
        mkdir_p(processing_action_folder)
        results = os.path.join(processing_action_folder, 'results')
        inputs = os.path.join(processing_action_folder, 'inputs')

        mkdir_p(results)
        mkdir_p(inputs)

        for key in ['merged_image_jpeg', 'merged_image', ]:
            files_to_download.append({
                'url': data[key],
                'filepath': os.path.join(results, get_filename_from_url(data[key]))
            })

        for input_object in data.get('inputs', []):
            files_to_download.append({
                'filepath': os.path.join(get_filename_from_url(input_object['file'])),
                'url': input_object['file']
            })

        with open(os.path.join(processing_action_folder, 'processing_action_%d.json' % action_id), 'w') as fout:
            json.dump(data, fout, indent=4)

        if verbose:
            click.echo('Downloading files for %s' % (action_id,))

    pool = Pool(concurrency)
    pool.map(download_file_from_dict, files_to_download)

    if verbose:
        click.echo('Processing download success')


def upload_preview_task(d):
    provider = d['provider']
    images_path = d['images_path']
    upc = d['upc']
    delete = d['delete']

    if delete:
        try:
            response = provider.delete_product_previews(upc)
            click.echo('Deleted %s previews for product %s' % (response['deleted'], upc,), err=True)
        except ProviderHTTPClientException:
            click.echo('Product with UPC %s does not exists' % upc, err=True)

    for image_path in images_path:
        with open(image_path, 'rb') as image:
            try:
                provider.upload_product_preview(upc, image)
            except ProviderHTTPClientException as e:
                click.echo("xxx Failed to send image: %s" % image_path)
                click.echo(str(e), err=True)
            click.echo('<< Uploaded product preview for %s from %s' % (upc, image_path), err=True)


@api.command()
@click.option('-d', '--delete', is_flag=True)
@click.option('-c', '--concurrency', type=int, default=4)
@click.argument('path', type=click.Path(exists=True), default=os.getcwd())
@click.pass_context
def upload_previews_from_folder(ctx, delete, concurrency, path):
    """
    Upload previews from file system to the server in parallel
    It has increased retries
    """
    provider = ctx.obj['provider']
    provider.retries = 5
    verbose = ctx.obj['verbose']

    tasks = []
    for label in os.listdir(path):
        upc_folder = os.path.join(path, label)
        if not os.path.isdir(upc_folder):
            continue

        if verbose:
            click.echo('Reading folder: %s' % upc_folder)
        task = {
            'provider': provider,
            'images_path': [],
            'delete': delete,
            'upc': label
        }

        if label.isdigit():
            for filename in os.listdir(upc_folder):
                image_path = os.path.join(upc_folder, filename)

                if os.path.isfile(image_path):
                    task['images_path'].append(image_path)

        if task['images_path']:
            tasks.append(task)

    if verbose:
        click.echo('Number of tasks: {}'.format(len(tasks)))

    p = Pool(concurrency)
    p.map(upload_preview_task, tasks)
    click.echo('Finished')


PROCESSING_STATUSES = {
    'created': 'action created',
    'done': 'done',
    'error': 'error',
    'interrupted': 'interrupted',
    'progress': "in progress",
}


@api.command()
@click.option('-s', '--store', type=click.INT)
@click.option('--status', type=click.Choice(PROCESSING_STATUSES.keys()))
@click.option('-p', '--page', type=click.INT, default=1)
@click.option('-r', '--page-size', type=click.INT, default=10)
@click.pass_context
def processing_actions(ctx, store, status, page, page_size):
    provider = ctx.obj['provider']
    verbose = ctx.obj['verbose']

    table = PrettyTable()
    table.field_names = ['#', 'id', 'store', 'user', 'status',
                         'created', 'last_requeue', 'last_requeue ago', 'time in queue']

    if verbose:
        click.echo('Getting list of processing actions')
    data = provider.get_processing_action_list(
        store,
        PROCESSING_STATUSES.get(status),
        page=page,
        page_size=page_size,
    )
    click.echo('Total results: %d' % len(data))
    if ctx.obj['format'] != 'id':
        format_processing_action_output(data, 'id')
    format_processing_action_output(data, ctx.obj['format'])
